"""
Example model
"""

from math import *
import os
import sys
import numpy

#name 
name = "RandomToOrientedCoreShellShellChains"

#title
title = "RandomToOrientedCoreShellShellChains"

#description
description = "Core-Shell-Shell Linear Chains Random-To-Oriented About X"

#parameters 
parameters = [ 
                ['core_sld', 'Inv. Ang^2', 6.9E-6, [-numpy.inf, numpy.inf], '', ''],
                ['magcore_sld', 'Inv. Ang^2', 1.5E-6, [-numpy.inf, numpy.inf], '', ''],
                ['shell1_sld', 'Inv. Ang^2', 1.0E-7, [-numpy.inf, numpy.inf], '', ''],
                ['shell2_sld', 'Inv. Ang^2', 1.0E-7, [-numpy.inf, numpy.inf], '', ''],
                ['solvent_sld', 'Inv. Ang^2', 6.35E-6, [-numpy.inf, numpy.inf], '', ''],
                ['volume_fraction', '', 0.0004, [0.0, numpy.inf], '', ''],
                ['normalization_radius', '', 125, [0.0, numpy.inf], '', ''],
                ['singlet_fraction', '', 0.5, [0.0, numpy.inf], '', ''],
                ['dimer_fraction', '', 0.3, [0.0, numpy.inf], '', ''],
                ['trimer_fraction', '', 0.2, [0.0, numpy.inf], '', ''],
                ['quadramer_fraction', '', 0.1, [0.0, numpy.inf], '', ''],
                ['pentamer_fraction', '', 0.1, [0.0, numpy.inf], '', ''],
                ['core_radius', 'Ang.', 125, [0.0, numpy.inf], 'volume', ''],
                ['shell1_thickness', 'Ang.', 25, [0.0, numpy.inf], 'volume', ''],
                ['shell2_thickness', 'Ang.', 25, [0.0, numpy.inf], 'volume', ''],
                ['length', 'Sphere to sphere length in Ang.', 350, [0.0, numpy.inf], 'volume', ''],
                ['m_var', '1=alongchain;2=alongchainskewtoX,3=alongfield', 1, [0, 3], '', ''],
                ['viewing_angle', 'w.r.t. applied field (degrees)', 0, [-90, 90], '', ''],
                ['sigma', 'Standard deviation of chain orientation (degrees)', 60, [0.1, 5000], '', ''],
                ['cross_section', '0=Unpol,1=UU,2=UD,3=DD,4=DU,5=DD-UU,6=|DD-UU|,7=DU+UD,8=DU-UD,9=|DU-UD|', 0.0, [0, 10], '', ''],
                ['porod_slope', '', -4.0, [-numpy.inf, numpy.inf], '', ''],
                ['porod_scale', '', 0.0, [0.0, numpy.inf], '', '']
             ]

form_volume = """
    return 1.0;
"""

def ER(*arg): 
    return 1.0 

Iq = """

    double Vol = M_4PI_3*pow(normalization_radius,3);
    if(Vol == 0){Vol = 1E-10;}
    double Q_X = q*cos(viewing_angle*M_PI_180);
    double Q_Y = q*sin(viewing_angle*M_PI_180);

    double GammaMatrix[3][3];
    double QA_hat= cos(viewing_angle*M_PI_180);
    double QB_hat= sin(viewing_angle*M_PI_180);
    double QC_hat= 0.0;
    GammaMatrix[0][0] = 1.0 - QA_hat*QA_hat; GammaMatrix[0][1] = -QA_hat*QB_hat; GammaMatrix[0][2] = -QA_hat*QC_hat;
    GammaMatrix[1][0] = 1.0 - QB_hat*QB_hat; GammaMatrix[1][1] = -QA_hat*QB_hat; GammaMatrix[1][2] = -QB_hat*QC_hat;
    GammaMatrix[2][0] = 1.0 - QC_hat*QC_hat; GammaMatrix[2][1] = -QA_hat*QC_hat; GammaMatrix[2][2] = -QB_hat*QC_hat;


    double AmpR1 = (sin(q*core_radius)/q-core_radius*cos(q*core_radius))/(q*q);
    double AmpR2 = (sin(q*(core_radius + shell1_thickness))/q-(core_radius + shell1_thickness)*cos(q*(core_radius + shell1_thickness)))/(q*q);
    double AmpR3 = (sin(q*(core_radius + shell1_thickness + shell2_thickness))/q-(core_radius + shell1_thickness + shell2_thickness)*cos(q*(core_radius + shell1_thickness + shell2_thickness)))/(q*q);
    double Amp = 4.0*M_PI*((core_sld-solvent_sld)*AmpR1 + (shell1_sld-solvent_sld)*(AmpR2-AmpR1) + (shell2_sld-solvent_sld)*(AmpR3-AmpR2));
    double MAmp = 4.0*M_PI*(magcore_sld)*AmpR1;

    const int a_steps = 17;
    const int b_steps = 12;
    double ChainProjX[20][20];//must be larger than a_steps, b_steps
    double ChainProjY[20][20];//must be larger than a_steps, b_steps
    double ChainProjZ[20][20];//must be larger than a_steps, b_steps
    double Norm = 0.0;
    double quad_place = 0.0;
    double angle_from_VA, angle_from_x, variable1, variable2;//angle is defined relative to viewing_angle (not x-axis)
    for(int a=0; a<a_steps; a++){//(a*2.0+1.0-90.0)*M_PI_180 for 0->90
        for(int b=0; b<b_steps; b++){//(b*45.0)*M_PI_180 for 0-> 12
            angle_from_VA = (a*5.0-90.0)*M_PI_180;
            double phi = (b*45.0)*M_PI_180;

		//********************************************
		//Rotate chains:
	    	double alpha[3], beta[3], gamma[3];
		alpha[0] = cos(viewing_angle*M_PI_180); alpha[1] = sin(viewing_angle*M_PI_180); alpha[2] = 0;
	    	beta[0] = cos(viewing_angle*M_PI_180+M_PI/2); beta[1] = sin(viewing_angle*M_PI_180+M_PI/2); beta[2] = 0;
	    	gamma[0] = 0.0; gamma[1] = 0.0; gamma[2] = 1.0;
		int points = 3;
		double LHS_Matrix[3][3];
		LHS_Matrix[0][0] = alpha[0]; LHS_Matrix[0][1] = alpha[1]; LHS_Matrix[0][2] = alpha[2];
		LHS_Matrix[1][0] = beta[0]; LHS_Matrix[1][1] = beta[1]; LHS_Matrix[1][2] = beta[2];
		LHS_Matrix[2][0] = gamma[0]; LHS_Matrix[2][1] = gamma[1]; LHS_Matrix[2][2] = gamma[2];
		double RHS_Matrix[3] = {cos(angle_from_VA), sin(angle_from_VA)*cos(phi), sin(angle_from_VA)*sin(phi)};
		double Alignment[3];

		if(alpha[0] <= 0.01 && alpha[0] >= -0.01){
		LHS_Matrix[1][0] = alpha[0]; LHS_Matrix[1][1] = alpha[1]; LHS_Matrix[1][2] = alpha[2];
		LHS_Matrix[0][0] = beta[0]; LHS_Matrix[0][1] = beta[1]; LHS_Matrix[0][2] = beta[2];
		RHS_Matrix[0] = sin(angle_from_VA)*cos(phi);
		RHS_Matrix[1] = cos(angle_from_VA);
		}
		

		//First row Gauss-Jordan ellimination:
		for(int col = 0; col<points-1; col++){
		for(int row = col+1; row<points; row++){
		for(int p=1; p<points-col; p++){
			LHS_Matrix[row][col+p] = LHS_Matrix[row][col+p] -(LHS_Matrix[row][col] / LHS_Matrix[col][col])*LHS_Matrix[col][col+p];
		}//end p loop
			RHS_Matrix[row] = RHS_Matrix[row] -(LHS_Matrix[row][col] / LHS_Matrix[col][col])*RHS_Matrix[col];
			LHS_Matrix[row][col] = LHS_Matrix[row][col] -(LHS_Matrix[row][col] / LHS_Matrix[col][col])*LHS_Matrix[col][col];
		}//end row loop
		}//end col loop

		//Additional rows of Gauss-Jordan ellimination:
		for(int col = 1; col<points; col++){
		for(int row = 0; row<col; row++){
		for(int p=1; p<points-col; p++){
			LHS_Matrix[row][col+p] = LHS_Matrix[row][col+p] -(LHS_Matrix[row][col] / LHS_Matrix[col][col])*LHS_Matrix[col][col+p];
		}//end p loop
			RHS_Matrix[row] = RHS_Matrix[row] -(LHS_Matrix[row][col] / LHS_Matrix[col][col])*RHS_Matrix[col];
			LHS_Matrix[row][col] = LHS_Matrix[row][col] -(LHS_Matrix[row][col] / LHS_Matrix[col][col])*LHS_Matrix[col][col];
		}//end row loop
		}//end col loop


		for(int p=0; p<points; p++){
		Alignment[p] = RHS_Matrix[p] / LHS_Matrix[p][p];
		}//end p loop

	    	double Unit = sqrt(Alignment[0]*Alignment[0] + Alignment[1]*Alignment[1] + Alignment[2]*Alignment[2]);
            	ChainProjX[a][b] = Alignment[0]/Unit;
            	ChainProjY[a][b] = Alignment[1]/Unit;
            	ChainProjZ[a][b] = Alignment[2]/Unit;
		//********************************************

	    //angle_from_x = viewing_angle + angle_from_VA;
	    angle_from_x = acos(ChainProjX[a][b]);
	    if(angle_from_x >= -M_PI && angle_from_x <= M_PI){quad_place = 0.0;}
	    if(angle_from_x < -M_PI){quad_place = M_PI;}
	    if(angle_from_x > M_PI){quad_place = -M_PI;}
            variable1 = (angle_from_x + quad_place - 0.0)/(sigma*M_PI_180);
            variable2 = sqrt(2.0*M_PI)*sigma*180.0/M_PI;
            Norm += sqrt(pow(sin(angle_from_VA),2))*(exp( -0.5*pow( variable1,2) ) / variable2);
        }
    }

    double CrossSection[4][5] = {0.0};
    double Nucl = 0.0;
    double Nucl2 = 0.0;

    double anglewt;
    for(int a=0; a<17; a++){
        for(int b=0; b<12; b++){
            angle_from_VA = (a*5.0-90.0)*M_PI_180;
            double phi = (b*45.0)*M_PI_180;
	    //angle_from_x = viewing_angle + angle_from_VA;
	    angle_from_x = acos(ChainProjX[a][b]);
	    if(angle_from_x >= -M_PI && angle_from_x <= M_PI){quad_place = 0.0;}
	    if(angle_from_x < -M_PI){quad_place = M_PI;}
	    if(angle_from_x > M_PI){quad_place = -M_PI;}
            variable1 = (angle_from_x + quad_place - 0.0)/(sigma*M_PI_180);
            variable2 = sqrt(2.0*M_PI)*sigma/M_PI_180;
            anglewt = sqrt(pow(sin(angle_from_VA),2))*(exp( -0.5*pow( variable1,2) ) / variable2)/Norm;

    	    double mag_a_real = 0.0;
    	    double mag_a_img = 0.0;
    	    double mag_b_real = 0.0;
    	    double mag_b_img = 0.0;
    	    double mag_c_real = 0.0;
    	    double mag_c_img = 0.0;
    	    double nucl_real = 0.0;
    	    double nucl_img = 0.0;
    	    double A_real = 0.0;
    	    double A_img = 0.0;
    	    double B_real = 0.0;
    	    double B_img = 0.0;
    	    double C_real = 0.0;
    	    double C_img = 0.0;

            for(int k=0; k<5; k++){
		double number = 1.0*k + 1.0;
                nucl_real += Amp*cos(k*length*(Q_X*ChainProjX[a][b] + Q_Y*ChainProjY[a][b]));
                nucl_img += Amp*sin(k*length*(Q_X*ChainProjX[a][b] + Q_Y*ChainProjY[a][b]));
		if(m_var <= 1.5){
		mag_a_real = MAmp*ChainProjX[a][b]*cos(k*length*(Q_X*ChainProjX[a][b] + Q_Y*ChainProjY[a][b]));
		mag_b_real = MAmp*ChainProjY[a][b]*cos(k*length*(Q_X*ChainProjX[a][b] + Q_Y*ChainProjY[a][b]));
		mag_c_real = MAmp*ChainProjZ[a][b]*cos(k*length*(Q_X*ChainProjX[a][b] + Q_Y*ChainProjY[a][b]));
		mag_a_img = MAmp*ChainProjX[a][b]*sin(k*length*(Q_X*ChainProjX[a][b] + Q_Y*ChainProjY[a][b]));
		mag_b_img = MAmp*ChainProjY[a][b]*sin(k*length*(Q_X*ChainProjX[a][b] + Q_Y*ChainProjY[a][b]));
		mag_c_img = MAmp*ChainProjZ[a][b]*sin(k*length*(Q_X*ChainProjX[a][b] + Q_Y*ChainProjY[a][b]));
		}
		if(m_var > 1.5 && m_var <= 2.5){
		mag_a_real = MAmp*sqrt(ChainProjX[a][b]*ChainProjX[a][b])*cos(k*length*(Q_X*ChainProjX[a][b] + Q_Y*ChainProjY[a][b]));
		mag_b_real = MAmp*ChainProjY[a][b]*cos(k*length*(Q_X*ChainProjX[a][b] + Q_Y*ChainProjY[a][b]));
		mag_c_real = MAmp*ChainProjZ[a][b]*cos(k*length*(Q_X*ChainProjX[a][b] + Q_Y*ChainProjY[a][b]));
		mag_a_img = MAmp*sqrt(ChainProjX[a][b]*ChainProjX[a][b])*sin(k*length*(Q_X*ChainProjX[a][b] + Q_Y*ChainProjY[a][b]));
		mag_b_img = MAmp*ChainProjY[a][b]*sin(k*length*(Q_X*ChainProjX[a][b] + Q_Y*ChainProjY[a][b]));
		mag_c_img = MAmp*ChainProjZ[a][b]*sin(k*length*(Q_X*ChainProjX[a][b] + Q_Y*ChainProjY[a][b]));
		}
		if(m_var > 2.5 && m_var <= 3.5){
		mag_a_real = MAmp*cos(k*length*(Q_X*ChainProjX[a][b] + Q_Y*ChainProjY[a][b]));
		mag_b_real = 0.0;
		mag_c_real = 0.0;
		mag_a_img = MAmp*sin(k*length*(Q_X*ChainProjX[a][b] + Q_Y*ChainProjY[a][b]));
		mag_b_img = 0.0;
		mag_c_img = 0.0;
		}
		A_real = GammaMatrix[0][0]*mag_a_real + GammaMatrix[0][1]*mag_b_real + GammaMatrix[0][2]*mag_c_real;
		A_img = GammaMatrix[0][0]*mag_a_img + GammaMatrix[0][1]*mag_b_img + GammaMatrix[0][2]*mag_c_img;
		B_real = GammaMatrix[1][0]*mag_b_real + GammaMatrix[1][1]*mag_a_real + GammaMatrix[1][2]*mag_c_real;
		B_img = GammaMatrix[1][0]*mag_b_img + GammaMatrix[1][1]*mag_a_img + GammaMatrix[1][2]*mag_c_img;
		C_real = GammaMatrix[2][0]*mag_c_real + GammaMatrix[2][1]*mag_a_real + GammaMatrix[2][2]*mag_b_real;
		C_img = GammaMatrix[2][0]*mag_c_img + GammaMatrix[2][1]*mag_a_img + GammaMatrix[2][2]*mag_b_img;
		CrossSection[0][k] += 0.5*anglewt*(pow(nucl_real-A_real,2)+pow(nucl_img-A_img,2))/(Vol*number);
		CrossSection[1][k] += 0.5*anglewt*(pow(-B_real+C_img,2)+pow(-B_img-C_real,2))/(Vol*number);
		CrossSection[2][k] += 0.5*anglewt*(pow(nucl_real+A_real,2)+pow(nucl_img+A_img,2))/(Vol*number);
		CrossSection[3][k] += 0.5*anglewt*(pow(-B_real-C_img,2)+pow(-B_img+C_real,2))/(Vol*number);
		Nucl2 += anglewt*(pow(nucl_real-A_real,2)+pow(nucl_img-A_img,2))/(Vol*number);
		Nucl += anglewt*(pow(nucl_real,2)+pow(nucl_img,2))/(Vol*number);

            }
       }
    }

    double Unpol = 0.0;
    double Intensity = 0.0;
    double FractionScale = singlet_fraction + dimer_fraction + trimer_fraction + quadramer_fraction + pentamer_fraction;
    if(FractionScale == 0){FractionScale = 1.0;}

    for(int n=0; n<4; n++){
    Unpol += singlet_fraction*CrossSection[n][0] + dimer_fraction*CrossSection[n][1] + trimer_fraction*CrossSection[n][2] + quadramer_fraction*CrossSection[n][3] + pentamer_fraction*CrossSection[n][4];
    }

    if(cross_section < 0.5){
    Intensity = Unpol;
    }

    if(cross_section >= 0.5 && cross_section <= 1.5){
    int n = 0;
    Intensity = singlet_fraction*CrossSection[n][0] + dimer_fraction*CrossSection[n][1] + trimer_fraction*CrossSection[n][2] + quadramer_fraction*CrossSection[n][3] + pentamer_fraction*CrossSection[n][4];
    }

    if(cross_section >= 1.5 && cross_section <= 2.5){
    int n = 1;
    Intensity = singlet_fraction*CrossSection[n][0] + dimer_fraction*CrossSection[n][1] + trimer_fraction*CrossSection[n][2] + quadramer_fraction*CrossSection[n][3] + pentamer_fraction*CrossSection[n][4];
    }

    if(cross_section >= 2.5 && cross_section <= 3.5){
    int n = 2;
    Intensity = singlet_fraction*CrossSection[n][0] + dimer_fraction*CrossSection[n][1] + trimer_fraction*CrossSection[n][2] + quadramer_fraction*CrossSection[n][3] + pentamer_fraction*CrossSection[n][4];
    }

    if(cross_section >= 3.5 && cross_section <= 4.5){
    int n = 3;
    Intensity = singlet_fraction*CrossSection[n][0] + dimer_fraction*CrossSection[n][1] + trimer_fraction*CrossSection[n][2] + quadramer_fraction*CrossSection[n][3] + pentamer_fraction*CrossSection[n][4];
    }

    if(cross_section >= 4.5 && cross_section <= 6.5){
    int n = 2;
    Intensity = singlet_fraction*CrossSection[n][0] + dimer_fraction*CrossSection[n][1] + trimer_fraction*CrossSection[n][2] + quadramer_fraction*CrossSection[n][3] + pentamer_fraction*CrossSection[n][4];
    n = 0;
    Intensity = Intensity - (singlet_fraction*CrossSection[n][0] + dimer_fraction*CrossSection[n][1] + trimer_fraction*CrossSection[n][2] + quadramer_fraction*CrossSection[n][3] + pentamer_fraction*CrossSection[n][4]);
	if(cross_section >= 5.5){double holder = pow(Intensity,2); Intensity = sqrt(holder);}
    }

    if(cross_section >= 6.5 && cross_section < 7.5){
    int n = 3;
    Intensity = singlet_fraction*CrossSection[n][0] + dimer_fraction*CrossSection[n][1] + trimer_fraction*CrossSection[n][2] + quadramer_fraction*CrossSection[n][3] + pentamer_fraction*CrossSection[n][4];
    n = 1;
    Intensity = Intensity + (singlet_fraction*CrossSection[n][0] + dimer_fraction*CrossSection[n][1] + trimer_fraction*CrossSection[n][2] + quadramer_fraction*CrossSection[n][3] + pentamer_fraction*CrossSection[n][4]);
    }

    if(cross_section >= 7.5){
    int n = 3;
    Intensity = singlet_fraction*CrossSection[n][0] + dimer_fraction*CrossSection[n][1] + trimer_fraction*CrossSection[n][2] + quadramer_fraction*CrossSection[n][3] + pentamer_fraction*CrossSection[n][4];
    n = 1;
    Intensity = Intensity - (singlet_fraction*CrossSection[n][0] + dimer_fraction*CrossSection[n][1] + trimer_fraction*CrossSection[n][2] + quadramer_fraction*CrossSection[n][3] + pentamer_fraction*CrossSection[n][4]);
	if(cross_section >= 8.5){double holder = pow(Intensity,2); Intensity = sqrt(holder);}
    }

    if(Intensity*1E6 < Unpol){
    Intensity = 0.0;
    }

    double PorodSlope = porod_scale*pow(q,porod_slope);

    return Intensity*volume_fraction*(1E8)/FractionScale + PorodSlope;

"""

Iqxy = """
    return Iq(sqrt(qx*qx+qy*qy), core_sld, magcore_sld, shell1_sld, shell2_sld, solvent_sld, volume_fraction, normalization_radius, singlet_fraction, dimer_fraction, trimer_fraction, quadramer_fraction, pentamer_fraction, core_radius, shell1_thickness, shell2_thickness, length, m_var, viewing_angle, sigma, cross_section, porod_slope, porod_scale);
"""
